
#include<iostream>
#include<occ-core/parser/Ptree.h>
#include<occ-core/parser/PtreeUtil.h>
#include<occ-core/parser/token-names.h>
#include<occ-core/parser/PtreeDeclarator.h>
#include"ClassBodyContextWalker.h"
#include<UML/Attribute.hpp>
#include<UML/Parameter.hpp>
#include<UML/Comment.hpp>
#include"PtreeAnalysen.h"

ClassBodyContextWalker::ClassBodyContextWalker(OMF::Model *model,UML::Class *umlclass,bool is_struct):model_(model),
       umlclass_(umlclass),
       currentvisibility_(is_struct ? UML::VisibilityKind::vk_public : UML::VisibilityKind::vk_private)
{
}

ClassBodyContextWalker::~ClassBodyContextWalker()
{
}

Opencxx::Ptree* ClassBodyContextWalker::TranslateAccessSpec(Opencxx::Ptree *aspec)
{
    std::string aspecstr(aspec->ToString());
    if(aspecstr.find("public")< std::string::npos)
    {
	currentvisibility_=UML::VisibilityKind::vk_public;
    }
    else if(aspecstr.find("protected")< std::string::npos)
    {
	currentvisibility_=UML::VisibilityKind::vk_protected;
    }
    else if(aspecstr.find("private")< std::string::npos)
    {
	currentvisibility_=UML::VisibilityKind::vk_private;
    }
    return aspec;
}

Opencxx::Ptree* ClassBodyContextWalker::TranslateDeclaration(Opencxx::Ptree* def)
{
    // SearchDeclarator(def);
    Opencxx::Ptree* decls = Opencxx::PtreeUtil::Third(def);
    if(decls->IsA(Opencxx::ntDeclarator))	// if it is a function
	return TranslateFunctionImplementation(def);
    else{
	char *cmdstr="";
	// if it is a function prototype or a variable declaration.
	Opencxx::Ptree* sspec = Opencxx::PtreeUtil::First(def);
	Opencxx::Ptree* tspec = Opencxx::PtreeUtil::Second(def);
	if(! decls->IsLeaf())	// if it is ";"
	{  
	    Opencxx::Ptree *inner=decls->Car();
	    if(inner->IsA(Opencxx::ntDeclarator))
	    {
		Opencxx::PtreeDeclarator *decl = (Opencxx::PtreeDeclarator *)inner;		      
		std::cerr << inner->DumpName()  << " -> " 
		      << decl->Name()->ToString()
		      << std::endl;
		Opencxx::Ptree *comment = decl->GetComments();
		std::cerr << "Comment:" << comment->ToString() << std::endl;
		cmdstr=comment->ToString();
	    }
	    else
	    {
		std::cerr << "NO NT-Declarator" << std::endl;
	    }
	    if(Opencxx::PtreeUtil::First(inner)->ToString()[0]=='*')
	    {
		// error in parser: returnvalue is a Pointer
		inner=inner->Cdr();
	    }
	    if(Opencxx::PtreeUtil::Length(inner)==1)
	    {
		Opencxx::Ptree *name = inner->Car();
		assembleVariable(sspec,tspec,name,cmdstr);
	    }
	    else if(Opencxx::PtreeUtil::Length(inner)>=4
		    && Opencxx::PtreeUtil::Second(inner)->ToString()[0]=='(')
	    {
		bool isabstract=(Opencxx::PtreeUtil::Length(inner)>4 
		   && Opencxx::PtreeUtil::Nth(inner,3)->ToString()[0]==')'
		   && Opencxx::PtreeUtil::Nth(inner,4)->ToString()[0]=='='
		   && Opencxx::PtreeUtil::Nth(inner,5)->ToString()[0]=='0');
		Opencxx::Ptree *name = Opencxx::PtreeUtil::First(inner);
		Opencxx::Ptree *args = Opencxx::PtreeUtil::Third(inner);
		assembleMethod(sspec,tspec,name,args,isabstract,cmdstr);
	    }
	    else
	    {
		std::cout << "\t\t" << inner->DumpName()
			  << " L=" << Opencxx::PtreeUtil::Length(inner)
			  << std::endl;
		std::cout << "\t\t";inner->Display(std::cout);
		std::cout << std::endl;
		DumpPtree(inner);
	    }
	}

	return def;
    }
}

void
ClassBodyContextWalker::assembleVariable(Opencxx::Ptree *storageattr,
					 Opencxx::Ptree *typ,
					 Opencxx::Ptree *name,
					 const char *comment)
{
    std::cout << "Variable: " << name->ToString() << std::endl;
    UML::Attribute *attribute = UML::Attribute::Proxy::instance().create();
    attribute->setName(UML::Name(name->ToString()));
    attribute->setVisibility(UML::VisibilityKind(currentvisibility_));
    if(strlen(comment)>0)
    {
	UML::Comment *uml_comment = UML::Comment::Proxy::instance().create();
	uml_comment->setBody(OMF::String(comment));
	attribute->addComment(uml_comment);
    }
    umlclass_->addOwnedElement(attribute);
    model_->identifyElement(attribute);
}

void
ClassBodyContextWalker::assembleMethod(Opencxx::Ptree *storageattr,
				       Opencxx::Ptree *rettyp,
				       Opencxx::Ptree *name,
				       Opencxx::Ptree *args,
				       bool isabstract,const char *commentstr)
{
    std::cout << (isabstract ? "abstract ":" ")
	      << "Operation:" << name->ToString() << std::endl;
    UML::Operation *op = UML::Operation::Proxy::instance().create();
    op->setName(UML::Name(name->ToString()));
    op->setVisibility(UML::VisibilityKind(currentvisibility_));
    op->setIsAbstract(OMF::Boolean(isabstract));
    if(strlen(commentstr)>0)
    {
	UML::Comment *uml_comment = UML::Comment::Proxy::instance().create();
	uml_comment->setBody(OMF::String(commentstr));
	op->addComment(uml_comment);
    }
    umlclass_->addOwnedElement(op);
    model_->identifyElement(op);
    assembleArgList(op,args);
}

void ClassBodyContextWalker::assembleArgList(UML::Operation *op,Opencxx::Ptree *args)
{
    if(args==0) return;
    std::cout << "\tArgumente:" << std::endl;
    Opencxx::Ptree *rest=args;
    while(rest!=0)
    {
	Opencxx::Ptree *p = rest->Car();
	if(p && p->ToString()[0]!=',')
	{
	    UML::Parameter *para = UML::Parameter::Proxy::instance().create();
	    p->Display();
	    Opencxx::Ptree *type = Opencxx::PtreeUtil::First(p);
	    Opencxx::Ptree *paraname =  Opencxx::PtreeUtil::Second(p);
	    std::cout << "Type: " << type->ToString()
		      << " Name: " << paraname->ToString() << std::endl;
	    para->setName(UML::Name(paraname->ToString()));
	    // para->setType(Classifier *value);
	    op->addParameter(para); // behavioral_Feature
	}
	rest=rest->Cdr();
    }
}

Opencxx::Ptree* ClassBodyContextWalker::TranslateFunctionImplementation(Opencxx::Ptree *decl)
{
    //std::cout << "TranslateFunctionImplementation(";
    //decl->Display(std::cout);
    //std::cout << std::endl;
    Opencxx::Ptree* sspec = Opencxx::PtreeUtil::First(decl);
    Opencxx::Ptree* tspec = Opencxx::PtreeUtil::Second(decl);
    Opencxx::Ptree* rumpf = Opencxx::PtreeUtil::Third(decl);
    Opencxx::Ptree *name = Opencxx::PtreeUtil::First(rumpf);
    Opencxx::Ptree *args = Opencxx::PtreeUtil::Third(rumpf);
    assembleMethod(sspec,tspec,name,args,false);
    return decl;
}

Opencxx::Ptree* ClassBodyContextWalker::TranslateDeclarators(Opencxx::Ptree *decl)
{
    std::cout << "TranslateDeclarators(";
    decl->Display(std::cout);
    std::cout << std::endl;
    return decl;
}


