#include"DefaultTranslatingWalker.h"
#include <occ-core/parser/Encoding.h>
#include <occ-core/parser/ErrorLog.h>
#include <occ-core/parser/PtreeArray.h>
#include <occ-core/parser/MopMsg.h>
#include <occ-core/parser/Parser.h>
#include <occ-core/parser/PtreeUtil.h>
#include <occ-core/parser/SourceLocation.h>
#include <occ-core/parser/TheErrorLog.h>
#include <occ-core/parser/deprecated.h>
#include <occ-core/parser/PtreeBlock.h>
#include <occ-core/parser/PtreeBrace.h>
#include <occ-core/parser/PtreeClassBody.h>
#include <occ-core/parser/PtreeClassSpec.h>
#include <occ-core/parser/PtreeDeclaration.h>
#include <occ-core/parser/PtreeDeclarator.h>
#include <occ-core/parser/PtreeExprStatement.h>
#include <occ-core/parser/PtreeFstyleCastExpr.h>
#include <occ-core/parser/PtreeLinkageSpec.h>
#include <occ-core/parser/PtreeNamespaceSpec.h>
#include <occ-core/parser/PtreeNamespaceAlias.h>
#include <occ-core/parser/PtreeTemplateDecl.h>
#include <occ-core/parser/PtreeTypedef.h>
#include <occ-core/parser/PtreeUsing.h>
#include <occ-core/parser/ptreeAll.h>
#include <occ-core/parser/token-names.h>
#include <occ-core/analyzer/PtreeTypeUtil.h>

using namespace Opencxx;

namespace DefaultLeaves
{
    Ptree* semicolon_t = new Leaf(";", 1);
}

/* stolen from occ-core/analyzer/Walker.cc */
Ptree* DefaultTranslatingWalker::TranslateBrace(Ptree *block)
{
    PtreeArray array;
    bool changed = false;
    Ptree* body = PtreeUtil::Second(block);
    Ptree* rest = body;
    while(rest != 0)
    {
        Ptree* p = rest->Car();
        Ptree* q = p->Translate(this);
        array.Append(q);
        if(p != q) {  changed = true; }
 
        rest = rest->Cdr();
    }
 
    if(changed)
    {
        return new PtreeBrace(PtreeUtil::First(block), array.All(),
                              PtreeUtil::Third(block));
    }
    else
    {
        return block;
    }

}


Ptree* DefaultTranslatingWalker::TranslateDeclaration(Ptree* def)
{
    Ptree* decls = PtreeUtil::Third(def);
    if(decls->IsA(ntDeclarator))	// if it is a function
	return TranslateFunctionImplementation(def);
    else{
	// if it is a function prototype or a variable declaration.
	Ptree* decls2;
	Ptree* sspec = PtreeUtil::First(def);
	Ptree* sspec2 = TranslateStorageSpecifiers(sspec);
	Ptree* tspec = PtreeUtil::Second(def);
	Ptree* tspec2 = TranslateTypespecifier(tspec);
	if(decls->IsLeaf())	// if it is ";"
	    decls2 = decls;
	else
	    decls2 = TranslateDeclarators(decls);

	if(sspec == sspec2 && tspec == tspec2 && decls == decls2)
	    return def;
	else if(decls2 == 0)
	    return new PtreeDeclaration(0, PtreeUtil::List(0,
							 DefaultLeaves::semicolon_t));
	else
	    return new PtreeDeclaration(sspec2,
					PtreeUtil::ShallowSubst(tspec2, tspec,
							    decls2, decls,
							    def->Cdr()));
    }
}

Ptree* DefaultTranslatingWalker::TranslateTypespecifier(Ptree* tspec)
{
    Ptree *class_spec, *class_spec2;

    class_spec = PtreeTypeUtil::GetClassOrEnumSpec(tspec);
    if(class_spec == 0)
	class_spec2 = 0;
    else
	class_spec2 = class_spec->Translate(this);

    if(class_spec == class_spec2)
	return tspec;
    else
	return PtreeUtil::ShallowSubst(class_spec2, class_spec, tspec);
}
