#include<occ-core/parser/Ptree.h>
#include<occ-core/parser/PtreeUtil.h>
#include<occ-core/parser/PtreeClassSpec.h>
#include"ExtractOMFWalker.h"
#include<UML/Model.hpp>
#include<UML/UMLPackage.hpp>
#include<UML/UMLLibrary.hpp>
#include<UML/Package.hpp>
#include<UML/Class.hpp>
#include<UML/Comment.hpp>
#include"ClassBodyContextWalker.h"
#include"PtreeAnalysen.h"

ExtractOMFWalker::ExtractOMFWalker()
{
    OMF::init();
    Model::init();
    UML::init();
    model_= new OMF::Model;
    model_->addMetamodel(&UML::UMLPackage::instance());
    current_scope_=0;
    currentScope_ = defaultScope_ = new Scope(this,(UML::Namespace*)0);
    TypedefStereotype_=0;
}

ExtractOMFWalker::~ExtractOMFWalker()
{
    delete defaultScope_;
}

ExtractOMFWalker::Scope *ExtractOMFWalker::getScope()
{
    return currentScope_;
}

void ExtractOMFWalker::Scope::addOwnedElement(UML::ModelElement *element)
{
    if(current_)
    {
	current_->addOwnedElement(element);
	walker_->model_->identifyElement(element);
    }
    else
    {
	walker_->model_->addElement(element);
    }
}

UML::Stereotype *ExtractOMFWalker::getTypedefStereotype()
{
    if(TypedefStereotype_==0)
    {
	TypedefStereotype_ = UML::Stereotype::Proxy::instance().create();
	TypedefStereotype_->setName(UML::Name("typedef"));
	//workaround
	model_->addElement(TypedefStereotype_);
    }
    return TypedefStereotype_;
}

Opencxx::Ptree* ExtractOMFWalker::TranslateNamespaceSpec(Opencxx::Ptree* def)
{

    Opencxx::Ptree* name = Opencxx::PtreeUtil::Second(def);
    Opencxx::Ptree* body = Opencxx::PtreeUtil::Third(def);
    UML::Package *umlpackage = UML::Package::Proxy::instance().create(); 
    umlpackage->setName(UML::Name(name->ToString()));
    getScope()->addOwnedElement(umlpackage);
    Scope scope(this,umlpackage);
    Opencxx::Ptree* body2 = body->Translate(this);
    return def;
}

Opencxx::Ptree* ExtractOMFWalker::TranslateClassSpec(Opencxx::Ptree* spec)
{
    Opencxx::Ptree* userkey;
    Opencxx::Ptree* class_def;
    const char *comment_str = "";
    std::cout << "TranslateClassSpec: " << spec->DumpName() << std::endl;
    Opencxx::PtreeClassSpec *classspec = dynamic_cast<Opencxx::PtreeClassSpec *>(spec);
    if(classspec!=0)
    {
	Opencxx::Ptree *comment = classspec->GetComments();
	comment_str = comment->ToString();
	std::cerr << " Comments: " << comment_str << std::endl;
    }
    else
    {
	std::cerr << "NO Classspec" << std::endl;
    }
    if(spec->Car()->IsLeaf())
    {
        userkey = 0;
        class_def = spec;
    }
    else
    {
        userkey = spec->Car();
	std::cerr << "Userkey: " << userkey->ToString() << std::endl;
        class_def = spec->Cdr();
    }

    if(Opencxx::PtreeUtil::Length(class_def) == 4) // "class", classname, inheritance, body
    {
	bool is_struct=false;
	//"class" is at 0
	Opencxx::Ptree *specifier = Opencxx::PtreeUtil::Nth(class_def,0);
	if(strcmp(specifier->ToString(),"struct")==0)
	{
	    std::cerr << "STRUCT" << std::endl;
	    is_struct=true;
	}
	Opencxx::Ptree *classname = Opencxx::PtreeUtil::Nth(class_def,1);
	Opencxx::Ptree *inheritance = Opencxx::PtreeUtil::Nth(class_def,2);
	Opencxx::Ptree *bodyblock = Opencxx::PtreeUtil::Nth(class_def,3);
	UML::Class *umlclass = UML::Class::Proxy::instance().create(); 
	umlclass->setName(UML::Name(classname->ToString()));
	if(0 && strlen(comment_str)>0) // comment out
	{
	    UML::Comment *uml_comment = UML::Comment::Proxy::instance().create();
	    uml_comment->setBody(OMF::String(comment_str));
	    umlclass->addComment(uml_comment); // cause crash in writer
	}
	getScope()->addOwnedElement(umlclass);
	
	Scope scope(this,umlclass);
	if(inheritance)
	{
	    std::cout << "Inheritance: " << inheritance->DumpName() << "\n";
	    inheritance->Display(std::cout);
	    std::cout << std::endl;
	    inheritance->Translate(this);
	}
	else
	{
	    std::cout << "Base class, no inheritance" << std::endl;
	}
	std::cout << "Body: " << bodyblock->DumpName() << "\n";
	/* -- eventual neu -- */
	Opencxx::Ptree *body = Opencxx::PtreeUtil::Second(bodyblock);
	Opencxx::Ptree* rest = body;
	ClassBodyContextWalker w(model_,umlclass,is_struct);
	while(rest != 0)
	{
	    Opencxx::Ptree* p = rest->Car();
	    std::cout << "\t " << p->DumpName() << std::endl;
	    Opencxx::Ptree* q = p->Translate(&w);
	    rest = rest->Cdr();
	}
    }
    else
    {
	std::cerr << "ClassSpec has length" << Opencxx::PtreeUtil::Length(class_def) 
		  << std::endl;
    }
    return spec;
}

Opencxx::Ptree* ExtractOMFWalker::TranslateTypedef(Opencxx::Ptree *spec)
{
    Opencxx::Ptree *tspec = Opencxx::PtreeUtil::Second(spec);
    Opencxx::Ptree *tname = Opencxx::PtreeUtil::Third(spec);
    const char *name;
    if(! tname->IsLeaf())
    {
	Opencxx::Ptree *pname;
	Opencxx::Ptree *tnameinner = tname->Car();
	if(Opencxx::PtreeUtil::Length(tnameinner)==2)
	{   // Sublimate Error: pointer is in name
	    pname=Opencxx::PtreeUtil::Second(tnameinner);
	    name=pname->ToString();
	}
	else
	{
	    name=tnameinner->ToString();
	}
    }
    else
    { // not reached
	name=tname->ToString();
    }
    std::cerr << "typedef " << name << std::endl;
    UML::Class *umlclass = UML::Class::Proxy::instance().create(); 
    umlclass->setName(UML::Name(name));
    umlclass->addStereotype(getTypedefStereotype());
    getScope()->addOwnedElement(umlclass);
    return spec;
}
