#include <iostream>
#include <fstream>
#include <occ-core/parser/Msg.h>
#include <occ-core/parser/ErrorLog.h>
#include <occ-core/parser/Parser.h>
#include <occ-core/parser/ProgramFile.h>
#include <occ-core/parser/ErrorLog.h>
#include <occ-core/parser/Lex.h>
#include <occ-core/parser/Ptree.h>
#include <occ-core/parser/Encoding.h>
#include "ExtractOMFWalker.h"

class MockErrorLog : public Opencxx::ErrorLog
{
public:
    void Report(const Opencxx::Msg& msg) 
    {
        msg.PrintOn(std::cerr);
        std::cerr << std::endl;
    }
};

int main(int argc,char **argv)
{
    MockErrorLog errorLog;
    Opencxx::Ptree *result;
    if(argc<=1)
    {
	std::cerr << "Usage: " << argv[0] << " source.ii" << std::endl;
	return -1;
    }
    std::string inputfilename(argv[1]);
    std::string::size_type pos=inputfilename.rfind('.');
    std::string outputfilename = inputfilename.substr(0,pos);
    outputfilename.append(".xmi");
    std::cerr << outputfilename << std::endl;
    std::ifstream inputFile(argv[1]);
    Opencxx::ProgramFile program(inputFile, argv[1]);
    Opencxx::Lex lexer(&program);
    Opencxx::Parser parser(&lexer, errorLog);
    ExtractOMFWalker eow;
    while(parser.rProgram(result))
    {
       result->Translate(&eow);
    }
    if (parser.SyntaxErrorsFound())
    {
	std::cerr << "ERROR: syntax error" << std::endl;
	return -1;
    }
    eow.getModel()->write(outputfilename.c_str());
    return 0;
}
